﻿using Microsoft.Extensions.Logging;
using ProjectAirBomber.CollectionGenericObjects;
using ProjectAirBomber.Drawnings;
using ProjectAirBomber.Exceptions;

namespace ProjectAirBomber;


/// <summary>
/// Форма работы с компанией и ее коллекцией
/// </summary>
public partial class FormBomberCollection : Form
{

    /// <summary>
    /// Хранилише коллекций
    /// </summary>
    private readonly StorageCollection<DrawningBomber> _storageCollection;

    /// <summary>
    /// Компания
    /// </summary>
    private AbstractCompany? _company = null;

    /// <summary>
    /// Логер
    /// </summary>
    private readonly ILogger _logger;

    /// <summary>
    /// Конструктор
    /// </summary>
    public FormBomberCollection(ILogger<FormBomberCollection> logger)
    {
        InitializeComponent();
        _storageCollection = new();
        _logger = logger;
    }


    /// <summary>
    /// Выбор компании
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ComboBoxSelectorCompany_SelectedIndexChanged(object sender, EventArgs e)
    {
        panelCompanyTools.Enabled = false;
    }

    /// <summary>
    /// Добавление военного самолета
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonAddBomber_Click(object sender, EventArgs e)
    {
        FormBomberConfig form = new FormBomberConfig();
        form.Show();
        form.AddEvent(SetBomber);
    }

    /// <summary>
    /// Добавление военного самолёта в коллекцию
    /// </summary>
    /// <param name="bomber"></param>
    private void SetBomber(DrawningBomber? bomber)
    {
        if (_company == null || bomber == null)
        {
            return;
        }

        try
        {
            int addingObject = (_company + bomber);
            MessageBox.Show("Объект добавлен");
            _logger.LogInformation($"Добавлен объект {bomber.GetDataForSave()}");
            pictureBox.Image = _company.Show();
        }
        catch (CollectionOverflowException ex)
        {
            MessageBox.Show("Не удалось добавить объект");
            _logger.LogError($"Не удалось добавить объект: {ex.Message}");
        }
    }


    /// <summary>
    /// Удаление объекта
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonRemoveBomber_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(maskedTextBoxPosition.Text) || _company == null)
        {
            _logger.LogError("Удаление объекта из несуществующей коллекции");
            return;
        }

        if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
        {
            return;
        }

        int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
        try
        {
            object decrementObject = _company - pos;
            MessageBox.Show("Объект удален");
            _logger.LogInformation($"Удален объект по позиции {pos}");
            pictureBox.Image = _company.Show();
        }
        catch (ObjectNotFoundException)
        {
            MessageBox.Show("Объект не найден");
            _logger.LogError($"Удаление не найденного объекта в позиции {pos} ");
        }
        catch (PositionOutOfCollectionException)
        {
            MessageBox.Show("Удаление вне рамках коллекции");
            _logger.LogError($"Удаление объекта за пределами коллекции {pos} ");
        }
    }

    /// <summary>
    /// Передача объекта в другую форму
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonGoToCheck_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }

        try
        {
            DrawningBomber? bomber = null;
            int counter = 100;
            while (bomber == null)
            {
                bomber = _company.GetRandomObject();
                counter--;
                if (counter <= 0)
                {
                    break;
                }
            }
            if (bomber == null)
            {
                return;
            }
            FormAirBomber form = new()
            {
                SetBomber = bomber
            };
            form.ShowDialog();
        }
        catch (ObjectNotFoundException)
        {
            _logger.LogError("Ошибка при передаче объекта на FormAirFighter");
        }
    }

    /// <summary>
    /// Перерисовка коллекции
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonRefresh_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }

        pictureBox.Image = _company.Show();
    }

    /// <summary>
    /// Добавление коллекции
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCollectionAdd_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(textBoxCollectionName.Text) ||
        (!radioButtonList.Checked && !radioButtonMassive.Checked))
        {
            MessageBox.Show("Не все данные заполнены", "Ошибка",
            MessageBoxButtons.OK, MessageBoxIcon.Error);
            return;
        }
        CollectionType collectionType = CollectionType.None;
        if (radioButtonMassive.Checked)
        {
            collectionType = CollectionType.Massive;
        }
        else if (radioButtonList.Checked)
        {
            collectionType = CollectionType.List;
        }
        _storageCollection.AddCollection(textBoxCollectionName.Text,
        collectionType);
        RerfreshListBoxItems();
    }
    /// <summary>
    /// Удаление коллекции
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCollectionDel_Click(object sender, EventArgs e)
    {
        if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
        {
            MessageBox.Show("Коллекция не выбрана");
            return;
        }

        if (MessageBox.Show("Удалить коллекцию?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
        {
            return;
        }
        _storageCollection.DelCollection(listBoxCollection.SelectedItem.ToString());
        RerfreshListBoxItems();
    }

    /// <summary>
    /// Обновление списка в listBoxCollection
    /// </summary>
    private void RerfreshListBoxItems()
    {
        listBoxCollection.Items.Clear();
        for (int i = 0; i < _storageCollection.Keys?.Count; ++i)
        {
            string? colName = _storageCollection.Keys?[i].Name;
            if (!string.IsNullOrEmpty(colName))
            {
                listBoxCollection.Items.Add(colName);
            }
        }
    }

    /// <summary>
    /// Создание компании
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCreateCompany_Click(object sender, EventArgs e)
    {
        if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
        {
            MessageBox.Show("Коллекция не выбрана");
            return;
        }
        ICollectionGenericObjects<DrawningBomber>? collection = _storageCollection[listBoxCollection.SelectedItem.ToString() ?? string.Empty];
        if (collection == null)
        {
            MessageBox.Show("Коллекция не проинициализирована");
            return;
        }
        switch (comboBoxSelectorCompany.Text)
        {
            case "Хранилище":
                _company = new BomberHungarService(pictureBox.Width, pictureBox.Height, collection);
                break;
        }
        panelCompanyTools.Enabled = true;
        RerfreshListBoxItems();
    }


    /// <summary>
	/// Обработка нажатия "Сохранение"
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
    private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
    {
        if (saveFileDialog.ShowDialog() == DialogResult.OK)
        {
            try
            {
                _storageCollection.SaveData(saveFileDialog.FileName);
                MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                _logger.LogInformation("Сохранение в файл: {filename}", saveFileDialog.FileName);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }
        }
    }


    /// <summary>
	/// Обработка нажатия "Загрузка"
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
    private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
    {
        if (openFileDialog.ShowDialog() == DialogResult.OK)
        {
            try
            {
                _storageCollection.LoadData(openFileDialog.FileName);
                MessageBox.Show("Загрузка прошла успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                _logger.LogInformation("Загрузка из файла: {filename}", openFileDialog.FileName);
                RerfreshListBoxItems();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }
        }
    }

    private void FormBomberCollection_Load(object sender, EventArgs e)
    {

    }

    /// <summary>
    /// Сортировка по типу
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonSortByType_Click(object sender, EventArgs e)
    {
        CompareBomber(new DrawingBomberCompareByType());
    }

    /// <summary>
    /// Сортировка по цвету
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonSortByColor_Click(object sender, EventArgs e)
    {
        CompareBomber(new DrawningBomberCompareByColor());
    }

    /// <summary>
    /// Сортировка по сравнителю
    /// </summary>
    /// <param name="comparer">Сравнитель объектов</param>
    private void CompareBomber(IComparer<DrawningBomber?> comparer)
    {
        if (_company == null)
        {
            return;
        }

        _company.Sort(comparer);
        pictureBox.Image = _company.Show();
    }
}